/*
 * Created on Jul 23, 2004
 */
package de.mdelab.instanceGraphEditor.ui.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.ui.IEditorPart;

import de.mdelab.instanceGraphEditor.ui.editor.InstanceGraphEditor;
import de.mdelab.instanceGraphEditor.ui.model.Graph;
import de.mdelab.instanceGraphEditor.ui.model.InstanceGraphFactory;


/**
 * Action to toggle the layout between manual and automatic
 * 
 */
public class FlyoutChangeLayoutAction extends Action
{

	IEditorPart editor;
	boolean checked;

	public FlyoutChangeLayoutAction(IEditorPart editor)
	{
		super("Automatic Layout", Action.AS_CHECK_BOX);
		this.editor = editor;
	}

	public void run()
	{
		if (editor instanceof InstanceGraphEditor)
		{
			Graph graph = ((InstanceGraphEditor) editor).getGraph();
			boolean isManual = graph.isLayoutManualDesired();
			graph.setLayoutManualDesired(!isManual);
			checked = !isManual;
			setChecked(checked);
		}
	}

	public boolean isChecked()
	{
		if (editor != null)
			return isChecked(editor);
		else
			return super.isChecked();
	}

	/**
	 * @see org.eclipse.jface.action.IAction#isChecked()
	 */
	public boolean isChecked(IEditorPart editor)
	{

		if (editor instanceof InstanceGraphEditor)
		{
			Graph graph = ((InstanceGraphEditor) editor).getGraph();
			boolean checkTrue = graph.isLayoutManualDesired();
			return (!checkTrue);
		}
		else
		{
			return false;
		}

	}

	public void setActiveEditor(IEditorPart editor)
	{
		this.editor = editor;
		boolean localChecked = isChecked(editor);
		
		//there appears to be a bug in the framework which necessitates this
		if (localChecked)
			firePropertyChange(CHECKED, Boolean.FALSE, Boolean.TRUE);
		else
			firePropertyChange(CHECKED, Boolean.TRUE, Boolean.FALSE);
	}

}