package de.mdelab.instanceGraphEditor.ui.commands;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;

import de.mdelab.instanceGraphEditor.ui.model.Link;
 
/**
 * Command used to delete a bendpoint from a {@link OPMLink}
 * This class is declared final since it has a very specific functionality.
 * @author vainolo
 */
public final class LinkDeleteBendpointCommand extends Command {
 
  /** Connection that contains the bendpoint. */
  private Link connection;
  /** Index where the bendpoint is located in the link's bendpoin list. */
  private int index;
  /** Point in the diagram where the bendpoint is located. */
  private Point location;
 
  /**
   * Only execute is link is not null and index is valid.
   */
  @Override public boolean canExecute() {
    return (connection != null) && (connection.getBendpoints().size() > index);
  }
  /**
   * Remove the bendpoint from the Connection.
   */
  @Override public void execute() {
    location = connection.getBendpoint(index);
    connection.removeBendpoint(index);
  }
 
  /**
   * Reinsert the bendpoint in the Connection.
   */
  @Override public void undo() {
	  connection.addBendpoint(index, location);
  }
 
  /**
   * Set the index of the bendpoint that should be removed.
   * @param index the index of the bendpoint to remove.
   */
  public void setIndex(final int index) {
    this.index = index;
  }
 
  /**
   * Set the link from which the bendpoint is removed.
   * @param link the link from which the bendpoint is removed.
   */
  public void setConnection(final Link connection) {
    this.connection = connection;
  }
}