package de.mdelab.instanceGraphEditor.ui.model;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Observable;

import org.eclipse.emf.ecore.EObject;

public class Graph extends Observable {

	private Hashtable<Object, Node> nodes;

	private Hashtable<EObject, List<EObject>> references;
	
	private boolean layoutManualDesired = false;
	
	public Graph() {

		nodes = new Hashtable<Object, Node>();
		references = new Hashtable<EObject, List<EObject>>();
	}
	
	public List<EObject> getReferences(EObject obj){
		
		List<EObject> refs;
		
		if((refs = references.get(obj)) == null){
			refs = new ArrayList<EObject>();
			references.put(obj, refs);
		}
		return refs;
	}

	public List<Node> getNodes() {
		return new ArrayList<Node>(nodes.values());
	}
	
	public void addNode(Node node) {
		nodes.put(node.getEObject(), node);
		setChanged();
		notifyObservers();
	}
	
	public void addNodes(List<Node> nodes) {
		for( Node n: nodes)
			addNode(n);
	}

	public void removeNode(Node node) {
		nodes.remove(node.getEObject());
		setChanged();
		notifyObservers();
	}
	
	public void removeNodes(List<Node> nodes) {
		for( Node n: nodes)
			removeNode(n);
	}
	
	public Node nodeIsCreated(Object obj){
		return nodes.get(obj) != null ? nodes.get(obj) : null;
	}
	
	public boolean isLayoutManualDesired(){
		return layoutManualDesired;
	}
	
	public void setLayoutManualDesired(boolean layoutManualDesired)
	{
		this.layoutManualDesired = layoutManualDesired;
		setChanged();
		notifyObservers();
	}
}