package de.mdelab.instanceGraphEditor.ui.parts;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.draw2d.AbsoluteBendpoint;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.graph.DirectedGraph;
import org.eclipse.draw2d.graph.Edge;
import org.eclipse.draw2d.graph.Node;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;

import de.mdelab.instanceGraphEditor.ui.model.Link;
import de.mdelab.instanceGraphEditor.ui.policies.LinkBendpointEditPolicy;
 

public class LinkEditPart extends AbstractConnectionEditPart implements Observer{
	
	public LinkEditPart(Link link) {
		setModel(link);
		link.addObserver(this);
	}

	@Override 
	protected void createEditPolicies() {
	    installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new ConnectionEndpointEditPolicy());
	    installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new LinkBendpointEditPolicy());
	  }

	protected IFigure createFigure() {
		
		// add arrow to the end point of the line
		PolygonDecoration decor = new PolygonDecoration();
		decor.setTemplate(PolygonDecoration.TRIANGLE_TIP);
		PolylineConnection conn = new PolylineConnection();
		conn.setTargetDecoration(decor);
		conn.setConnectionRouter(new BendpointConnectionRouter());

		// add Label to the adge
		ConnectionEndpointLocator relationshipLocator = new ConnectionEndpointLocator(conn, false);
		relationshipLocator.setVDistance(5);
		relationshipLocator.setUDistance(20);
		Label relationshipLabel = new Label(((Link)getModel()).getClassifier());
		conn.add(relationshipLabel, relationshipLocator);
		return conn;
	}
	
	@Override 
	protected void refreshVisuals() {
		Connection connection = getConnectionFigure();
	    List<Point> modelConstraint = ((Link)getModel()).getBendpoints();
	    List<AbsoluteBendpoint> figureConstraint = new ArrayList<AbsoluteBendpoint>();
	    for (Point p : modelConstraint) {
	      figureConstraint.add(new AbsoluteBendpoint(p));
	    }
	    connection.setRoutingConstraint(figureConstraint);
	  }

	public void contributeToGraph(DirectedGraph graph, Map map) {

		Node source = (Node) map.get(getSource());
		Node target = (Node) map.get(getTarget());
		if (source != null && target != null) {
			Edge e = new Edge(this, source, target);
			graph.edges.add(e);
		}
	}

	//Observe changes on the model object
	@Override
	public void update(Observable o, Object arg) {
		refreshVisuals();
		
	}
}