package de.mdelab.instanceGraphEditor.ui.policies;

import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ComponentEditPolicy;

import de.mdelab.instanceGraphEditor.ui.actions.ExpandNodeAction;
import de.mdelab.instanceGraphEditor.ui.actions.HideChildrenAction;
import de.mdelab.instanceGraphEditor.ui.actions.HideNodeAction;
import de.mdelab.instanceGraphEditor.ui.actions.PrintOutNodeAction;
import de.mdelab.instanceGraphEditor.ui.commands.NodeExpandCommand;
import de.mdelab.instanceGraphEditor.ui.commands.NodeHideChildrenCommand;
import de.mdelab.instanceGraphEditor.ui.commands.NodeHideCommand;
import de.mdelab.instanceGraphEditor.ui.commands.PrintOutNodeCommand;
import de.mdelab.instanceGraphEditor.ui.model.Node;
import de.mdelab.instanceGraphEditor.ui.parts.NodesEditPart;

public class NodesEditPolicy extends ComponentEditPolicy {

    private static final int INSETS = 20;    
    
    private Command createExpandChildrenCommand() {
        NodesEditPart host = (NodesEditPart) getHost();
        Node node = (Node) host.getModel();

        NodeExpandCommand command = new NodeExpandCommand();
        command.setNode(node);
        return command;
    }
    
    private Command createHideChildrenCommand() {
        NodesEditPart host = (NodesEditPart) getHost();
        Node node = (Node) host.getModel();

        NodeHideChildrenCommand command = new NodeHideChildrenCommand();
        command.setNode(node);
        return command;
    }
    
    private Command createHideNodeCommand() {
        NodesEditPart host = (NodesEditPart) getHost();
        Node node = (Node) host.getModel();

        NodeHideCommand command = new NodeHideCommand();
        command.setNode(node);
        return command;
    }
    
    private Command createPrintOutNodeCommand() {
        NodesEditPart host = (NodesEditPart) getHost();
        Node node = (Node) host.getModel();

        PrintOutNodeCommand command = new PrintOutNodeCommand();
        command.setNode(node);
        return command;
    }
    
    @Override
    public Command getCommand(Request request) {
        if(request.getType().equals(ExpandNodeAction.REQ_EXPAND_NODE)) {
            return createExpandChildrenCommand();
        } else if(request.getType().equals(HideChildrenAction.REQ_HIDE_CHILDREN)) {
            return createHideChildrenCommand();
        } else if(request.getType().equals(HideNodeAction.REQ_HIDE_NODE)) {
            return createHideNodeCommand();
        } else if(request.getType().equals(PrintOutNodeAction.REQ_PRINT_OUT)) {
            return createPrintOutNodeCommand();
        }
        return super.getCommand(request);
    }    
}
