package de.mdelab.sdm.interpreter.code.debug.ui;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.DebugException;
import org.eclipse.emf.ecore.EObject;

import de.mdelab.sdm.interpreter.core.debug.client.DebugEvent;
import de.mdelab.sdm.interpreter.core.debug.client.ExecutionFinishedEvent;
import de.mdelab.sdm.interpreter.core.debug.client.ExecutionResumedEvent;
import de.mdelab.sdm.interpreter.core.debug.client.ExecutionSuspendedEvent;
import de.mdelab.sdm.interpreter.core.debug.client.ExecutionTerminatedEvent;
import de.mdelab.sdm.interpreter.core.debug.client.SDDebugClient;

public class SDDebugEventListenerJob<StoryDiagramElement extends EObject> extends Job {
	
	private final SDDebugTarget<StoryDiagramElement, ?>	debugTarget;
	private final SDDebugClient<StoryDiagramElement, ?>	debugClient;

	public SDDebugEventListenerJob(	final SDDebugTarget<StoryDiagramElement, ?> debugTarget,
									final SDDebugClient<StoryDiagramElement, ?> debugClient ) {
		super( "Story Diagram Debug Event Listener" );
		
		this.setSystem( true );

		assert debugTarget != null;
		assert debugClient != null;

		this.debugTarget = debugTarget;
		this.debugClient = debugClient;
	}

	@Override
	protected IStatus run(IProgressMonitor monitor)	{
		while ( !debugTarget.isTerminated() ) {
			try {
				final DebugEvent debugEvent = debugClient.waitForDebugEvent( null );
				
				// DB: Need a distinct API for events received from server because the fire methods should
				// also be called by methods called from debugger buttons otherwise the buttons statuses are
				// not updated
				if (debugEvent instanceof ExecutionResumedEvent) {
					//debugTarget.fireResumeEvent( org.eclipse.debug.core.DebugEvent.UNSPECIFIED );
					debugTarget.debuggerResumed();
				}
				else if ( debugEvent instanceof ExecutionSuspendedEvent ) {
					if ( ( (ExecutionSuspendedEvent) debugEvent ).getBreakpointUUID() != null ) {
						//debugTarget.fireSuspendEvent(org.eclipse.debug.core.DebugEvent.BREAKPOINT);
						debugTarget.debuggerSuspendedAtBreakpoint();
					}
					else {
						debugTarget.debuggerSuspended();
//						debugTarget.fireSuspendEvent(org.eclipse.debug.core.DebugEvent.UNSPECIFIED);
					}
				}
				else if (debugEvent instanceof ExecutionFinishedEvent || debugEvent instanceof ExecutionTerminatedEvent ) {
					try {
						debugTarget.debuggerTerminated();
					} 
					catch ( final DebugException p_ex ) {
						return p_ex.getStatus();
					}
				}
//				else if (debugEvent instanceof ExecutionTerminatedEvent) {
//					this.debugTarget.fireTerminateEvent();
//				}
				else {
					throw new UnsupportedOperationException();
				}
			}
			catch (InterruptedException e) {
				throw new UnsupportedOperationException();
			}
		}

		return Status.OK_STATUS;
	}
}
