package de.mdelab.sdm.interpreter.code.debug.ui;

import org.eclipse.core.resources.IMarkerDelta;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.model.DebugElement;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.debug.core.model.IMemoryBlock;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.ISourceLocator;
import org.eclipse.debug.core.model.IThread;

public abstract class SDRunTarget extends DebugElement implements IDebugTarget {

	private final ILaunch launch;
	
	private final IThread[] debugThreads;
	
	private boolean terminated;

	public SDRunTarget(	final ILaunch p_launch ) {
		super( null );

		assert p_launch != null;

		launch = p_launch;
		
		debugThreads = new SDDebugThread[ 0 ];
		
		terminated = false;
		p_launch.setSourceLocator( createSDDebugSourceLocator() );
	}
	
	protected abstract ISourceLocator createSDDebugSourceLocator();

	@Override
	public boolean canTerminate() {
		return !terminated;
	}

	@Override
	public boolean isTerminated() {
		return terminated;
	}

	@Override
	public void terminate()
	throws DebugException {
		terminated = true;

		fireTerminateEvent();
	}

	@Override
	public boolean canResume() {
		return false;
	}

	@Override
	public boolean canSuspend()	{
		return false;
	}

	@Override
	public boolean isSuspended() {
		return false;
	}

	@Override
	public void resume()
	throws DebugException {
	}

	@Override
	public void suspend()
	throws DebugException {
	}
	
	@Override
	public void breakpointAdded(IBreakpoint breakpoint) {
	}

	@Override
	public void breakpointRemoved(IBreakpoint breakpoint, IMarkerDelta delta) {
	}

	@Override
	public void breakpointChanged(IBreakpoint breakpoint, IMarkerDelta delta) {
	}

	@Override
	public boolean canDisconnect() {
		return false;
	}

	@Override
	public void disconnect()
	throws DebugException {
	}

	@Override
	public boolean isDisconnected()	{
		return false;
	}

	@Override
	public boolean supportsStorageRetrieval() {
		return false;
	}

	@Override
	public IProcess getProcess() {
		return null;
	}

	@Override
	public IThread[] getThreads()
	throws DebugException {
		return debugThreads;
	}

	@Override
	public boolean hasThreads()
	throws DebugException {
		return false;
	}

	@Override
	public String getName()
	throws DebugException {
		return SDDebugUiConstants.SD_RUN_TARGET_NAME;
	}

	@Override
	public boolean supportsBreakpoint(IBreakpoint breakpoint) {
		return false;
	}

	@Override
	public ILaunch getLaunch() {
		return launch;
	}

	@Override
	public IMemoryBlock getMemoryBlock(long startAddress, long length)
	throws DebugException {
		return null;
	}

	@Override
	public String getModelIdentifier() {
		return SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER;
	}
}
