package de.mdelab.sdm.interpreter.code.debug.ui.launcher;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.emf.common.ui.dialogs.ResourceDialog;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;

import de.mdelab.sdm.interpreter.code.debug.ui.EMFResourceSelectionDialog;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.actparameter.ISDTableActivityParametersViewerListener;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.actparameter.SDActivityParameterTableViewer;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.actparameter.SDTableActivityParameter;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.instmodels.IInputModelsTableViewerListener;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.instmodels.InstanceModelRow;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.instmodels.InstanceModelTableViewer;
import de.mdelab.sdm.interpreter.core.executionTrace.ExecutionTracePackage;

public abstract class SDInterpreterLaunchMainTab<Activity> extends AbstractLaunchConfigurationTab {
	
	// DB: Issue #1038: Allow to provide many input instance models
	private InstanceModelTableViewer instanceModelTableViewer 	= null;
	//private ListViewer	instanceModelURIsViewer				= null;
//	private Text		textInstanceModelURI				= null;

	private Text		textStoryDiagramURI					= null;
	private Button		buttonBrowseStoryDiagramURI 		= null;

	private Combo		comboStoryActivity					= null;

//	private Text		textOutputInstanceModelURI			= null;
//	private Button		buttonBrowseOutputInstanceModelURI 	= null;

	private Text		textExecutionTraceModelURI	= null;
	private Button		buttonBrowseExecutionTrace 	= null;

	private Button checkboxLogOutput = null;
	
	private SDActivityParameterTableViewer<Activity, ? extends SDTableActivityParameter<?, ?>, ?> activityParametersViewer = null;
	
	private final ResourceSet resSet = new ResourceSetImpl();
	
	/**
	 * Indicates that the launch configuration should not be saved. This is because control listeners
	 * will be fired when displaying the data which triggers performApply before allthe control data
	 * has been set
	 */
	private boolean shouldPerformApply = true;
	
	public void createControl( final Composite p_parent ) {
		Composite compositeMain = createComposite( p_parent, 1, 1, GridData.FILL_HORIZONTAL );
		setControl( compositeMain );
		
		createInterpreterLaunchGroup( compositeMain );
	}
	
	protected void setLaunchGroupEnabled( final boolean pb_enabled ) {
		// DB: Issue #1038: Allow to provide many input instance models
		instanceModelTableViewer.setEnabled( pb_enabled );
		//buttonBrowseInstanceModelURI.setEnabled( pb_enabled );

		textStoryDiagramURI.setEnabled( pb_enabled );
		buttonBrowseStoryDiagramURI.setEnabled( pb_enabled );
		comboStoryActivity.setEnabled( pb_enabled );
		activityParametersViewer.setEnabled( pb_enabled  );
		
		//textOutputInstanceModelURI.setEnabled( pb_enabled );
		//buttonBrowseOutputInstanceModelURI.setEnabled( pb_enabled );
		
		textExecutionTraceModelURI.setEnabled( pb_enabled );
		buttonBrowseExecutionTrace.setEnabled( pb_enabled );
		checkboxLogOutput.setEnabled( pb_enabled );
	}
	
	@Override
	public void dispose() {
		if ( instanceModelTableViewer != null ) {
			instanceModelTableViewer.dispose();
		}
		
		if ( activityParametersViewer != null ) {
			activityParametersViewer.dispose();
		}

		super.dispose();
	}

	private void createInterpreterLaunchGroup( final Composite p_compositeMain ) {
		final Group interpreterLauncherGroup = createGroup( p_compositeMain, "Story Diagram Interpreter Launch Parameters", 1 );

		// instance group
		createInstanceModelGroup( interpreterLauncherGroup );

		// story diagram group
		createStoryDiagramModelGroup( interpreterLauncherGroup );
		
		createActivityParametersGroup( interpreterLauncherGroup );

		// output group
		//createOutputModelGroup( interpreterLauncherGroup );

		// execution trace model
		createExecutionTraceModelGroup( interpreterLauncherGroup );
	}
	
	protected abstract SDActivityParameterTableViewer<Activity, ? extends SDTableActivityParameter<?, ?>, ?> createActivityParameterTableViewer( Composite p_parent );

	private void createActivityParametersGroup( final Composite p_compositeMain ) {
		final Group group = createGroup( p_compositeMain, "Activity Parameters", 2 );
		activityParametersViewer = createActivityParameterTableViewer( group );
		activityParametersViewer.addViewerListener( new ISDTableActivityParametersViewerListener() {
			
			@Override
			public void rowObjectUpdated( final SDTableActivityParameter<?, ?> p_parameter ) {
				setDirty( true );
				
				updateLaunchConfigurationDialog();
			}
			
			@Override
			public void rowObjectRemoved( final SDTableActivityParameter<?, ?> p_parameter) {
			}
			
			@Override
			public void rowObjectAdded( final SDTableActivityParameter<?, ?> p_parameter) {
			}
		} );
	}

	private void createExecutionTraceModelGroup( final Composite p_compositeMain ) {
		final Group groupExecutionTrace = createGroup( p_compositeMain, "Execution Trace Model", 2 );
		
		textExecutionTraceModelURI = createSingleText( groupExecutionTrace, "Enter the path of the execution trace model. This can be left empty if you do not want to save an execution trace." );
		textExecutionTraceModelURI.addModifyListener( new ModifyListener() {
			
			@Override
			public void modifyText(ModifyEvent e) {
				final Resource sdRes = loadStoryDiagram();
				
				if ( sdRes != null ) {
					setDirty( true );
					
					updateLaunchConfigurationDialog();
				}
			}
		} );

		buttonBrowseExecutionTrace = createButton( groupExecutionTrace, "Browse...", "Browse for an execution trace model." );
		buttonBrowseExecutionTrace.addSelectionListener( new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e)	{
				widgetSelected(e);
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				// DB: Issue 1038: Handle cancel
				if ( promtUpdateValue( "Select Execution Trace Model", textExecutionTraceModelURI, ExecutionTracePackage.eNAME, false ) ) {
					setDirty( true );
	
					updateLaunchConfigurationDialog();
				}
			}
		} );

		checkboxLogOutput = createCheckButton( groupExecutionTrace, "Log Execution Output to Console" ); 
		checkboxLogOutput.addSelectionListener( new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e)	{
				widgetSelected(e);
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				setDirty( true );
				
				updateLaunchConfigurationDialog();
			}
		} );
	}
	
	// DB: Issue 1038: Allow several input instance models to be used as input	
	private boolean promtUpdateValue( 	final String p_title,
										final Text p_text,
										final String p_fileExtensionFilter,
										boolean pb_manyFiles ) {
		final String uris = promtUpdateValue( p_title, p_fileExtensionFilter, pb_manyFiles );
		
		if ( uris == null ) {
			return false;
		}
		
		p_text.setText( uris );
		
		return true;
	}

	private String promtUpdateValue( 	final String p_title,
										final String p_fileExtensionFilter,
										boolean pb_manyFiles ) {
		final int style = pb_manyFiles ? SWT.OPEN | SWT.MULTI : SWT.OPEN | SWT.SINGLE;
		final ResourceDialog dialog = new EMFResourceSelectionDialog( getShell(), p_title, style, p_fileExtensionFilter == null ? null : new String[]{ p_fileExtensionFilter } );
		
		if ( dialog.open() == ResourceDialog.OK ) {
			return dialog.getURIText();
		}
		
		return null;
	}
//
//	private void createOutputModelGroup(Composite compositeMain) {
//		Group groupOutput = createGroup(compositeMain, "Output Model", 2);
//
//		textOutputInstanceModelURI = createSingleText(groupOutput, "Enter the path of the output model.");
//		textOutputInstanceModelURI.addModifyListener( new ModifyListener() {
//			
//			@Override
//			public void modifyText(ModifyEvent e) {
//				final Resource sdRes = loadStoryDiagram();
//				
//				if ( sdRes != null ) {
//					setDirty( true );
//					
//					updateLaunchConfigurationDialog();
//				}
//			}
//		} );

//		buttonBrowseOutputInstanceModelURI = createButton( groupOutput, "Browse...", "Browse for an output model." );
//		buttonBrowseOutputInstanceModelURI.addSelectionListener(new SelectionListener() {
//
//			@Override
//			public void widgetDefaultSelected(SelectionEvent e)	{
//				widgetSelected(e);
//			}
//
//			@Override
//			public void widgetSelected(SelectionEvent e) {
//				
//				// Issue #1038: Handel cancel
//				if ( promtUpdateValue( "Select Output Model", textOutputInstanceModelURI, null, false ) ) {
//					setDirty( true );
//	
//					updateLaunchConfigurationDialog();
//				}
//			}
//		} );
//	}
	
	protected abstract String getSDFileExtension();

	private void createStoryDiagramModelGroup(Composite compositeMain) {
		Group groupStoryDiagram = createGroup(compositeMain, "Activity to Execute", 2);

		textStoryDiagramURI = createSingleText(groupStoryDiagram, "Choose a story diagram file." );
		textStoryDiagramURI.addModifyListener( new ModifyListener() {
			
			@Override
			public void modifyText(ModifyEvent e) {
				final Resource sdRes = loadStoryDiagram();
				
				if ( sdRes != null ) {
					loadComboActivities( sdRes, 0 );
					updateActivityParameters();
					setDirty( true );
					
					updateLaunchConfigurationDialog();
				}
			}
		} );

		buttonBrowseStoryDiagramURI = createButton( groupStoryDiagram, "Browse...", "Browse for a story diagram." );
		buttonBrowseStoryDiagramURI.addSelectionListener( new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e)	{
				widgetSelected(e);
			}

			public void widgetSelected(SelectionEvent e) {
				if ( promtUpdateValue( "Select Story Diagram", textStoryDiagramURI, getSDFileExtension(), false ) ) {
					final Resource sdRes = loadStoryDiagram();
					
					if ( sdRes != null ) {
						loadComboActivities( sdRes, 0 );
						updateActivityParameters();
						setDirty( true );
						
						updateLaunchConfigurationDialog();
					}
				}
			}
		} );

		comboStoryActivity = createCombo( groupStoryDiagram );
		comboStoryActivity.addSelectionListener( new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e)	{
				widgetSelected(e);
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				updateActivityParameters();
				setDirty( true );
				
				updateLaunchConfigurationDialog();
			}
		} );
	}
	
	@SuppressWarnings("unchecked")
	private Activity getSelectedActivity() {
		final Resource storyDiagRes = loadStoryDiagram();
		
		if ( storyDiagRes == null || storyDiagRes.getContents().isEmpty() ) {
			return null; 
		}
		
		final int comboIndex = comboStoryActivity.getSelectionIndex();
		
		final List<Activity> activities = new ArrayList<Activity>();
		
		for ( final EObject element : storyDiagRes.getContents() ) {
			if ( isActivity( element ) ) {
				activities.add( (Activity) element );
			}
		}
		
		if ( comboIndex >= 0 && comboIndex < activities.size() ) {
			return activities.get( comboIndex );
		}
		
		return null;
	}
	
	private Resource loadStoryDiagram() {
		final String resUriStr = textStoryDiagramURI.getText();
		
		if ( resUriStr == null || resUriStr.isEmpty() ) {
			return null;
		}
		
		try {
			return resSet.getResource( URI.createURI( resUriStr ), true );
		}
		catch( final RuntimeException p_ex ) {
			MessageDialog.openError( getShell(), "Error Loading Story Diagram", "The story diagram '" + resUriStr + "' could not be loaded." );
				
			return null;
		}
	}
	
	// Issue #1038
	private Set<Resource> loadInputInstanceModels() {
		final Set<Resource> resources = new HashSet<Resource>();
			
		for ( final InstanceModelRow row : instanceModelTableViewer.getTableData().getRowObjects() ) {
			final URI uri = row.getInputInstanceModel();
			try {
				resources.add( resSet.getResource( uri, true ) );
			}
			catch( final RuntimeException p_ex ) {
				MessageDialog.openError( getShell(), "Error Loading an Instance Model", "The model '" + uri + "' could not be loaded: " + p_ex.getLocalizedMessage() );
				p_ex.printStackTrace();
				
				return null;
			}
		}	
		
		return resources;
	}

	private void updateActivityParameters() {
		activityParametersViewer.initData( getSelectedActivity(), loadInputInstanceModels() );
	}
	
	private void createInstanceModelGroup(Composite compositeMain) {
		final Group groupInstanceModel = createGroup( compositeMain, "Instance Model(s) on which the Story Diagram will Operate", 2 );

		instanceModelTableViewer = new InstanceModelTableViewer( groupInstanceModel );
		instanceModelTableViewer.addViewerListener( new IInputModelsTableViewerListener() {
			
			@Override
			public void rowObjectUpdated( final InstanceModelRow p_parameter ) {
				setDirty( true );
				updateLaunchConfigurationDialog();
			}
			
			@Override
			public void rowObjectRemoved( final InstanceModelRow p_parameter) {
				updateActivityParameters();				
				setDirty( true );
				updateLaunchConfigurationDialog();
			}
			
			@Override
			public void rowObjectAdded( final InstanceModelRow p_parameter) {
				updateActivityParameters();				
				setDirty( true );
				updateLaunchConfigurationDialog();
			}

			@Override
			public void inputModelUpdated(InstanceModelRow p_modelRow) {
				updateActivityParameters();				
			}
		} );
	}

	private Combo createCombo(Composite parent)
	{
		Combo combo = createCombo(parent, SWT.READ_ONLY, 1, null);
		combo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
		return combo;
	}

	private Button createButton(Composite parent, String text, String toolTipText)
	{
		Button b = createPushButton(parent, text, null);
		b.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false, 1, 1));
		b.setToolTipText(toolTipText);
		return b;
	}

	public String getName()	{
		return "Main";
	}

	@Override
	public void initializeFrom( final ILaunchConfiguration p_configuration ) {
		shouldPerformApply = false;

		// Output model URI
		//initializeTextFieldFromAttribute( textOutputInstanceModelURI, ISDLaunchUIConstants.ATTR_OUTPUT_MODEL_URI, p_configuration, "" );

		// Execution trace model URI
		initializeTextFieldFromAttribute( textExecutionTraceModelURI, ISDLaunchUIConstants.ATTR_EXECUTION_TRACE_URI, p_configuration, "" );
		
		try	{
			checkboxLogOutput.setSelection( p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_LOG_OUTPUT, false ) );
	
			// Instance Model URI and Instance Objects
			final Map<String, String> urisStr = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_INSTANCE_MODEL_URI, Collections.<String, String> emptyMap() );
			
			final List<InstanceModelRow> rows = new ArrayList<InstanceModelRow>();
			
			for ( final Map.Entry<String, String> entry : urisStr.entrySet() ) {
				final URI inputUri = URI.createURI( entry.getKey() );
				final URI outUri;
				final String outUriStr = entry.getValue();
				
				if ( outUriStr == null || urisStr.isEmpty() ) {
					outUri = null;
				}
				else {
					outUri = URI.createURI( outUriStr );
				}
				
				rows.add( new InstanceModelRow( inputUri, outUri ) );
			}
			
			instanceModelTableViewer.initData( rows );
			//initializeTextFieldFromAttribute( textInstanceModelURI, ISDLaunchUIConstants.ATTR_INSTANCE_MODEL_URI, p_configuration, "" );
	
			// StoryDiagram URI and story activities
			initializeTextFieldFromAttribute( textStoryDiagramURI, ISDLaunchUIConstants.ATTR_STORY_DIAGRAM_URI, p_configuration, "" );
			final String storyDiagUriStr = textStoryDiagramURI.getText();

			if ( !storyDiagUriStr.isEmpty() ) {
				final Resource storyDiagRes = loadStoryDiagram();

				if ( storyDiagRes != null ) {
					if ( !storyDiagRes.getContents().isEmpty() ) {
						final int index = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_STORY_ACTIVITY_SELECTION_INDEX, 0 );

						if ( index < storyDiagRes.getContents().size() ) {
							loadComboActivities( storyDiagRes, index );
							final Activity activity = getSelectedActivity();

							final Set<Resource> instModelRes = loadInputInstanceModels();
							
							if ( instModelRes != null ) {
								final Map<String, String> parameters = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_STORY_ACTIVITY_PARAMETERS, Collections.<String, String> emptyMap() );
								activityParametersViewer.initData( activity, instModelRes, parameters );
							}
						}
					}
				}
			}
		}
		catch ( final CoreException p_ex ) {
			throw new RuntimeException( p_ex );
		}
		finally {
			shouldPerformApply = true;
		}
	}
	
	private void loadComboActivities( 	final Resource p_res,
										final int pi_indexSel ) {
		comboStoryActivity.removeAll();

		for ( final EObject object : p_res.getContents() ) {
			if ( isActivity( object ) ) {
				@SuppressWarnings("unchecked")
				final Activity act = (Activity) object;
				final String actName = getName( act );
				comboStoryActivity.add( actName == null ? "Unnamed" : actName );
			}
		}

		comboStoryActivity.select( pi_indexSel );
	}
	
	protected abstract String getName( Activity p_activity );

	@Override
	public void performApply( final ILaunchConfigurationWorkingCopy p_configuration ) {
		if ( shouldPerformApply ) {
			p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_INSTANCE_MODEL_URI, instanceModelTableViewer.getDataString() );
			//p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_OUTPUT_MODEL_URI, textOutputInstanceModelURI.getText() );
			p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_STORY_DIAGRAM_URI, textStoryDiagramURI.getText() );
			p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_STORY_ACTIVITY_SELECTION_INDEX, comboStoryActivity.getSelectionIndex() );
			p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_STORY_ACTIVITY_PARAMETERS, activityParametersViewer.getDataString() );
			p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_LOG_OUTPUT, checkboxLogOutput.getSelection() );
			p_configuration.setAttribute( ISDLaunchUIConstants.ATTR_EXECUTION_TRACE_URI, textExecutionTraceModelURI.getText() );
		}
	}

	@Override
	public void setDefaults( final ILaunchConfigurationWorkingCopy p_configuration ) {
	}
	
	private Set<Resource> validateInstanceModel( final ILaunchConfiguration p_configuration ) 
	throws CoreException {
		final Map<String, String> instModelUrisStr = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_INSTANCE_MODEL_URI, Collections.<String, String> emptyMap() );

		if ( instModelUrisStr.isEmpty() ) {
			setErrorMessage( "There is no instance model provided." );
			
			return null;
		}
		
		final Set<Resource> resources = new HashSet<Resource>();
		
		for ( final Map.Entry<String, String> entry : instModelUrisStr.entrySet() ) {
			final String uriStr = entry.getKey();
			final URI uri = URI.createURI( uriStr );
	
			if ( resSet.getURIConverter().exists( uri, null ) ) {
				try {
					final Resource instModelRes = resSet.getResource( uri, true );
	
					if ( instModelRes.getContents().isEmpty() )	{
						setErrorMessage( "The instance model is empty." );
		
						return null;
					}

					resources.add( instModelRes );
		
					final String outModelUri = entry.getValue();
					
					if ( !validateOutputModel( outModelUri ) ) {
						return null;
					}
				}
				catch( final RuntimeException p_ex ) {
					setErrorMessage( "Error Loading Instance Model '" + uriStr + "': " + p_ex.getLocalizedMessage() );
					
					return null;
				}
			}
			else {
				setErrorMessage( "Instance model resource '" + uriStr + "' does not exist." );
				
				return null;
			}
		}

		return resources;
	}
	
	protected abstract boolean validateActivityParameters( 	ILaunchConfiguration p_configuration,
															Activity p_activity )
	throws CoreException;

	@SuppressWarnings("unchecked")
	private Activity validateActivity( final ILaunchConfiguration p_configuration ) 
	throws CoreException {
		final String storyDiagResUriStr = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_STORY_DIAGRAM_URI, "" );

		if  ( storyDiagResUriStr.isEmpty() ) {
			setErrorMessage( "The story diagram resource URI is not set." );
			
			return null;
		}

		final URI storyDiagResUri = URI.createURI( storyDiagResUriStr );
		
		if ( resSet.getURIConverter().exists( storyDiagResUri, null ) ) {
			try {
				Resource storyDiagRes = resSet.getResource( storyDiagResUri, true );
	
				if ( storyDiagRes.getContents().isEmpty() )	{
					setErrorMessage( "The story diagram resource does not contain any activity." );
					
					return null;
				}
	
				final int selectedActivityIndex = p_configuration.getAttribute( ISDLaunchUIConstants.ATTR_STORY_ACTIVITY_SELECTION_INDEX, -1 );
	
				if ( selectedActivityIndex < 0 || selectedActivityIndex >= storyDiagRes.getContents().size() ) {
					setErrorMessage( "Please select an activity to execute." );
					
					return null;
				}
				
				final EObject element = storyDiagRes.getContents().get( selectedActivityIndex );
				
				if ( isActivity( element ) ) {
					return (Activity) element;
				}
	
				setErrorMessage( "The selected element in the story diagram is not an activity." );
				
				return null;
			}
			catch( final RuntimeException p_ex ) {
				setErrorMessage( "Error Loading Story Diagram '" + storyDiagResUriStr + "': " + p_ex.getLocalizedMessage() );
				
				return null;
			}
		}

		setErrorMessage( "The story diagram does not exists." );
			
		return null;
	}
	
	protected abstract boolean isActivity( EObject p_object );
	
	private boolean validateOutputModel( final String p_outModelResUri )
	throws CoreException {
		if ( p_outModelResUri != null && p_outModelResUri.isEmpty() ) {
			setErrorMessage("An output model must be specified.");

			return false;
		}
		
		return true;
	}
	
	private boolean validateExecutionTrace( final ILaunchConfiguration p_configuration )
	throws CoreException {
		return true;
	}

	@Override
	public boolean isValid( final ILaunchConfiguration p_configuration ) {
		if ( super.isValid( p_configuration ) ) {
			setErrorMessage( null );
			
			try {
				final Activity selectedActivity = validateActivity( p_configuration );
				
				if ( selectedActivity != null ) {
					final Set<Resource> instModelRes = validateInstanceModel( p_configuration );
					
					if ( instModelRes != null && !instModelRes.isEmpty() ) {
						if ( validateExecutionTrace( p_configuration ) ) {
							return validateActivityParameters( p_configuration, selectedActivity );
						}
					}
				}
			}
			catch ( final CoreException p_ex ) {
				throw new RuntimeException( p_ex );
			}
		}
		
		return false;
	}

	protected Composite createComposite(Composite parent, int columns, int hspan, int fill)
	{
		Composite g = new Composite(parent, SWT.NONE);
		g.setLayout(new GridLayout(columns, false));
		g.setFont(parent.getFont());
		GridData gd = new GridData(fill);
		gd.horizontalSpan = hspan;
		g.setLayoutData(gd);
		return g;
	}

	protected Group createGroup(Composite parent, String text, int columns)
	{
		Group g = new Group(parent, SWT.NONE);
		g.setLayout(new GridLayout(columns, false));
		g.setText(text);
		g.setFont(parent.getFont());
		GridData gd = new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1);
		gd.widthHint = 640;
		g.setLayoutData(gd);

		return g;
	}

	protected Text createSingleText(Composite parent, String toolTipText) {
		Text t = new Text(parent, SWT.SINGLE | SWT.BORDER);
		t.setFont(parent.getFont());
		t.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));
		t.setToolTipText(toolTipText);

		return t;
	}

	protected static Combo createCombo(Composite parent, int style, int hspan, String[] items)
	{
		Combo c = new Combo(parent, style);
		c.setFont(parent.getFont());
		GridData gd = new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1);
		gd.horizontalSpan = hspan;
		c.setLayoutData(gd);
		if (items != null)
		{
			c.setItems(items);
		}
		c.select(0);
		return c;
	}

	protected void initializeTextFieldFromAttribute( 	final Text p_textfield, 
														final String p_attrName,
														final ILaunchConfiguration p_configuration,
														final String p_defaultVal ) {
		try {
			p_textfield.setText( p_configuration.getAttribute( p_attrName, p_defaultVal ) );
		} 
		catch ( final CoreException p_ex ) {
			throw new RuntimeException( p_ex );
		}
	}
}
