package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;

public abstract class TableViewerComparator<ColumnType> extends ViewerComparator  {

	protected ColumnType criteria;

	protected boolean descending;
	
	protected final ILabelProvider labelProvider;

	/**
	 * Creates a resource sorter that will use the given sort criteria.
	 *
	 * @param criteria the sort criterion to use: one of <code>NAME</code> or 
	 *   <code>TYPE</code>
	 */
	public TableViewerComparator( final ILabelProvider p_labelProvider ) {
		super();
		
		descending = true;
		
		labelProvider = p_labelProvider;
	}
	
	public int getDirection() {
	    return descending ? SWT.DOWN : SWT.UP;
	}
	
	public void setColumn( final ColumnType p_column ) {
		if ( p_column == criteria ) {
			//Same column as last sort; toggle the direction
			descending = !descending;
		}
		else {
			// New column; do an ascending sort
			criteria = p_column;
			descending = true;
		}
	}	

	/**
	 * Returns the sort criteria of this this sorter.
	 *
	 * @return the sort criterion
	 */
	public ColumnType getCriteria() {
		return criteria;
	}
}
