package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.instmodels;

import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.Viewer;

import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.TableViewerComparator;

public class InstanceModelComparator extends TableViewerComparator<InstanceModelsColumnType>  {

	/**
	 * Creates a resource sorter that will use the given sort criteria.
	 *
	 * @param criteria the sort criterion to use: one of <code>NAME</code> or 
	 *   <code>TYPE</code>
	 */
	public InstanceModelComparator( final ILabelProvider p_labelProvider ) {
		super( p_labelProvider );
		
		criteria = InstanceModelsColumnType.INPUT;
	}
	
	/* (non-Javadoc)
	 * Method declared on ViewerSorter.
	 */
	public int compare(Viewer viewer, Object o1, Object o2) {
		final InstanceModelRow param1 = (InstanceModelRow) o1;
		final InstanceModelRow param2 = (InstanceModelRow) o2;
		
		int result = 0;

		switch ( criteria ) {
			case INPUT :
				result = compare( param1.getInputInstanceModel(), param2.getInputInstanceModel() );
				
				break;
			
			case OUTPUT :
				result = compare( param1.getOutputInstanceModel(), param2.getOutputInstanceModel() );
				
				break;
			
			default:
				throw new IllegalArgumentException( "Unknown type of column: " + criteria );
		}
		
		if ( descending ) {
			result = - result;
		}
		
		return result;
	}

	protected int compare( 	final URI p_param1,
							final URI p_param2 ) {
		if ( p_param1 == null ) {
			if ( p_param2 == null ) {
				return 0;
			}
			
			return 1;
		}
		
		if ( p_param2 == null ) {			
			return - 1;
		}

		return p_param1.toString().compareTo( p_param2.toString() );
	}
}
