package de.mdelab.sdm.interpreter.code.debug.ui.variables;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IVariable;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.provider.EcoreItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;

import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugTarget;

public class SDEObjectValue<StoryDiagramElement extends EObject> extends SDValue<StoryDiagramElement> {
	
	private final EObject	eObject;
	private final EClass	eClass;
	private final AdapterFactoryLabelProvider emfLabelProvider;

	public SDEObjectValue(SDDebugTarget<StoryDiagramElement, ?> target, EClass eClass, EObject eObject)	{
		super(target);

		assert eClass != null;

		this.eClass = eClass;
		this.eObject = eObject;
		
		emfLabelProvider = new AdapterFactoryLabelProvider( new EcoreItemProviderAdapterFactory() );
	}
	
	@Override
	public void finalize()
	throws Throwable {
		emfLabelProvider.dispose();
		
		super.finalize();
	}

	@Override
	public String getReferenceTypeName() throws DebugException
	{
		// TODO
		throw new UnsupportedOperationException();
	}

	@Override
	public String getValueString()
	throws DebugException {
//		StringBuilder stringBuilder = new StringBuilder();
//		stringBuilder.append(this.eClass.getName());
//
//		stringBuilder.append(" (");
//
//		if (this.eObject == null)
//		{
//			stringBuilder.append("null");
//		}
//		else
//		{
//			stringBuilder.append(this.eObject.toString());
//		}
//
//		stringBuilder.append(")");
//
//		return stringBuilder.toString();
		return emfLabelProvider.getText( eObject );
	}

	@Override
	public IVariable[] getVariables() throws DebugException
	{
		List<SDVariable<StoryDiagramElement>> variables = new LinkedList<SDVariable<StoryDiagramElement>>();
		final SDDebugTarget<StoryDiagramElement, ?> debugTarget = getDebugTarget();

		for (EStructuralFeature feature : this.eClass.getEAllStructuralFeatures()) {
			if (feature.eClass() == EcorePackage.Literals.EATTRIBUTE)
			{
				if (!feature.isMany())
				{
					Object o = this.eObject.eGet(feature);

					variables.add(new SDPrimitiveFeatureVariable<StoryDiagramElement>( debugTarget, feature.getName(),
							this.eObject, feature, (EDataType) feature.getEType(), o));
				}
				else
				{
					variables
							.add(new SDListVariable<StoryDiagramElement>( debugTarget, feature.getName(), this.eObject, feature));
				}
			}
			else if (feature.eClass() == EcorePackage.Literals.EREFERENCE)
			{
				if (!feature.isMany())
				{
					EObject e = (EObject) this.eObject.eGet(feature);

					variables.add(new SDEObjectFeatureVariable<StoryDiagramElement>( debugTarget, feature.getName(),
							this.eObject, feature, (EClass) feature.getEType(), e));
				}
				else
				{
					variables
							.add(new SDListVariable<StoryDiagramElement>( debugTarget, feature.getName(), this.eObject, feature));
				}
			}
			else {
				throw new UnsupportedOperationException();
			}
		}

		return variables.toArray(new IVariable[variables.size()]);
	}

	@Override
	public boolean hasVariables()
	throws DebugException {
		return this.eObject != null && !this.eClass.getEStructuralFeatures().isEmpty();
	}
}
